# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; withclass department(osv.osv):
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import osv,fields
from osv.orm import except_orm
import datetime
import time
import netsvc
from tools.translate import _
from dateutil.relativedelta import relativedelta
import logging

NUMBER_OF_UNUSED_MONTHS_BEFORE_EXPIRY = 36

logger = logging.getLogger(__name__)

class res_company(osv.osv):
    _inherit = 'res.company'
    _columns = {
        'sepa_creditor_identifier': fields.char('SEPA Creditor Identifier', size=35,
            help="Enter the Creditor Identifier that has been attributed to your company to make SEPA Direct Debits. This identifier is composed of :\n- your country ISO code (2 letters)\n- a 2-digits checkum\n- a 3-letters business code\n- a country-specific identifier"),
        'original_creditor_identifier': fields.char('Original Creditor Identifier', size=70),
        'sepa_iban': fields.char('SEPA Direct Debit IBAN',size=35,help="IBAN of the account number used to collect direct debit"),
        'sepa_bic': fields.char('SEPA Direct Debit BIC',size=8,help="BIC of the account number used to collect direct debit"),
    }

    def is_sepa_creditor_identifier_valid(self, cr, uid, sepa_creditor_identifier, context=None):
        """Check if SEPA Creditor Identifier is valid
        @param sepa_creditor_identifier: SEPA Creditor Identifier as str
            or unicode
        @return: True if valid, False otherwise
        """
        if not isinstance(sepa_creditor_identifier, (str, unicode)):
            return False
        try:
            sci_str = str(sepa_creditor_identifier)
        except:
            logger.warning(
                "SEPA Creditor ID should contain only ASCII caracters.")
            return False
        sci = sci_str.lower()
        if len(sci) < 9:
            return False
        before_replacement = sci[7:] + sci[0:2] + '00'
        logger.debug(
            "SEPA ID check before_replacement = %s" % before_replacement)
        after_replacement = ''
        for char in before_replacement:
            if char.isalpha():
                after_replacement += str(ord(char)-87)
            else:
                after_replacement += char
        logger.debug(
            "SEPA ID check after_replacement = %s" % after_replacement)
        if int(sci[2:4]) == (98 - (int(after_replacement) % 97)):
            return True
        else:
            return False

    def _check_sepa_creditor_identifier(self, cr, uid, ids):
        for company in self.browse(cr, uid, ids):
            if company.sepa_creditor_identifier:
                if not self.is_sepa_creditor_identifier_valid(
                        cr, uid, company.sepa_creditor_identifier):
                    return False
        return True

    _constraints = [
        (_check_sepa_creditor_identifier,
            "Invalid SEPA Creditor Identifier.",
            ['sepa_creditor_identifier']),
    ]
res_company()

class sdd_mandate(osv.osv):
    '''SEPA Direct Debit Mandate'''
    _name = 'cci.sdd.mandate'
    _description = __doc__
    #_rec_name = 'unique_mandate_reference'
    #_inherit = ['mail.thread']
    _order = 'signature_date desc'
    _columns = {
        'partner_bank_id': fields.many2one('res.partner.bank', 'Bank Account'),
        'partner_id': fields.related(
            'partner_bank_id', 'partner_id', type='many2one',
            relation='res.partner', string='Partner'),
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'unique_mandate_reference': fields.char('Unique Mandate Reference', size=35, readonly=True),
        'type': fields.selection([
            ('recurrent', 'Recurrent'),
            ('oneoff', 'One-Off'),
            ], 'Type of Mandate', required=True),
        'recurrent_sequence_type': fields.selection([
            ('first', 'First'),
            ('recurring', 'Recurring'),
            ('final', 'Final'),
            ], 'Sequence Type for Next Debit',
            help="This field is only used for Recurrent mandates, not for One-Off mandates."),
        'signature_date': fields.date('Date of Signature of the Mandate'),
        'scan': fields.binary('Scan of the Mandate'),
        'local_instrument': fields.selection([('CORE','Core'),('B2B','Business-to-Business')],'Local Instrument'),
        'last_debit_date': fields.date('Date of the Last Debit', readonly=True),
        'sepa_migrated': fields.boolean(
            'Migrated to SEPA', track_visibility='onchange',
            help="If this field is not active, the mandate section of the "
            "next direct debit file that include this mandate will contain "
            "the 'Original Mandate Identification' and the 'Original "
            "Creditor Scheme Identification'. This is required in a few "
            "countries (Belgium for instance), but not in all countries. "
            "If this is not required in your country, you should keep this "
            "field always active."),
        'original_mandate_identification': fields.char(
            'Original Mandate Identification', size=35,
            track_visibility='onchange',
            help="When the field 'Migrated to SEPA' is not active, this "
            "field will be used as the Original Mandate Identification in "
            "the Direct Debit file."),
        'state': fields.selection([
            ('draft', 'Draft'),
            ('valid', 'Valid'),
            ('expired', 'Expired'),
            ('cancel', 'Cancelled'),
            ], 'Status',
            help="Only valid mandates can be used in a payment line. A cancelled mandate is a mandate that has been cancelled by the customer. A one-off mandate expires after its first use. A recurrent mandate expires after it's final use or if it hasn't been used for 36 months."),
        'note': fields.text('Note'),
        }

    _defaults = {
        'unique_mandate_reference': lambda self, cr, uid, ctx:
            self.pool.get('ir.sequence').get(cr, uid, 'sdd.mandate.reference'),
        'sepa_migrated': lambda *a: True,
        'state': lambda *a: 'draft',
        'local_instrument': lambda *a: 'CORE',
        'type': lambda *a: 'recurrent',
        'recurrent_sequence_type': lambda *a: 'first',
    }

    _sql_constraints = [(
        'mandate_ref_company_uniq',
        'unique(unique_mandate_reference, company_id)',
        'A Mandate with the same reference already exists for this company !'
        )]

    def _check_sdd_mandate(self, cr, uid, ids, context=None):
        for mandate in self.read(cr, uid, ids, [
                'last_debit_date', 'signature_date',
                'unique_mandate_reference', 'state', 'partner_bank_id',
                'type', 'recurrent_sequence_type',
                ], context=context):
            if (mandate['signature_date'] and
                    mandate['signature_date'] >
                    datetime.datetime.today().strftime('%Y-%m-%d')):
                raise osv.except_osv(
                    _('Error:'),
                    _("The date of signature of mandate '%s' is in the future!")
                    % mandate['unique_mandate_reference'])
            if mandate['state'] == 'valid' and not mandate['signature_date']:
                raise osv.except_osv(
                    _('Error:'),
                    _("Cannot validate the mandate '%s' without a date of signature.")
                    % mandate['unique_mandate_reference'])
            if mandate['state'] == 'valid' and not mandate['partner_bank_id']:
                raise osv.except_osv(
                    _('Error:'),
                    _("Cannot validate the mandate '%s' because it is not attached to a bank account.")
                    % mandate['unique_mandate_reference'])

            if (mandate['signature_date'] and mandate['last_debit_date'] and
                    mandate['signature_date'] > mandate['last_debit_date']):
                raise osv.except_osv(
                    _('Error:'),
                    _("The mandate '%s' can't have a date of last debit before the date of signature.")
                    % mandate['unique_mandate_reference'])
            if (mandate['type'] == 'recurrent'
                    and not mandate['recurrent_sequence_type']):
                raise osv.except_osv(
                    _('Error:'),
                    _("The recurrent mandate '%s' must have a sequence type.")
                    % mandate['unique_mandate_reference'])
        return True

    _constraints = [
        (_check_sdd_mandate, "Error msg in raise", [
            'last_debit_date', 'signature_date', 'state', 'partner_bank_id',
            'type', 'recurrent_sequence_type',
            ]),
    ]

    def mandate_type_change(self, cr, uid, ids, type):
        if type == 'recurrent':
            recurrent_sequence_type = 'first'
        else:
            recurrent_sequence_type = False
        res = {'value': {'recurrent_sequence_type': recurrent_sequence_type}}
        return res

    def mandate_partner_bank_change(
            self, cr, uid, ids, partner_bank_id, type, recurrent_sequence_type,
            last_debit_date, state):
        if (state == 'valid' and partner_bank_id
                and type == 'recurrent'
                and recurrent_sequence_type != 'first'):
            return {
                'value': {'recurrent_sequence_type': 'first'},
                'warning': {
                    'title': _('Mandate update'),
                    'message': _("As you changed the bank account attached to this mandate, the 'Sequence Type' has been set back to 'First'."),
                    }
                }
        else:
            return True

    def validate(self, cr, uid, ids, context=None):
        to_validate_ids = []
        for mandate in self.browse(cr, uid, ids, context=context):
            assert mandate.state == 'draft', 'Mandate should be in draft state'
            to_validate_ids.append(mandate.id)
        self.write(
            cr, uid, to_validate_ids, {'state': 'valid'}, context=context)
        return True

    def cancel(self, cr, uid, ids, context=None):
        to_cancel_ids = []
        for mandate in self.browse(cr, uid, ids, context=context):
            assert mandate.state in ('draft', 'valid'),\
                'Mandate should be in draft or valid state'
            to_cancel_ids.append(mandate.id)
        self.write(
            cr, uid, to_cancel_ids, {'state': 'cancel'}, context=context)
        return True

    def _check_valid_mandates(self,cr,uid,context=None):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'SEPA_CheckingValidSEPAData')
        final_result = True
        # check value sepa_migrated and BIC value on linked accounts
        invalid_mandate_ids = self.search(cr,uid,[('sepa_migrated','=',False),('original_mandate_identification','=',False),('state','=','valid')])
        if invalid_mandate_ids:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nThe following SDD Mandate come from DOM80 and have no Original Mandate Identification: %s' % invalid_mandate_ids)
            final_result = False
        invalid_mandate_ids = self.search(cr,uid,[('sepa_migrated','=',True),('original_mandate_identification','<>',False),('state','=','valid'),('recurrent_sequence_type','=','first')])
        if invalid_mandate_ids:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nThe following SDD Mandate are not from DOM80 but have an Original Mandate Identification: %s' % invalid_mandate_ids)
            final_result = False
        current_mandate_ids = self.search(cr,uid,[('state','=','valid')])
        if current_mandate_ids:
            current_mandates = self.browse(cr,uid,current_mandate_ids)
            invalid_mandate_ids = []
            for mandate in current_mandates:
                if not (mandate.partner_bank_id and mandate.partner_bank_id.bank and mandate.partner_bank_id.bank.bic):
                    invalid_mandate_ids.append(mandate.id)
        if invalid_mandate_ids:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nNo BIC on linked bank account of these mandates : %s' % invalid_mandate_ids)
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _sdd_mandate_set_state_to_expired(self, cr, uid, context=None):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'SEPA_CheckingExpiredMandates')
        final_result = True
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking if some recurring SEPA Direct Debit Mandates have expired\n')
        expire_limit_date = datetime.datetime.today() + relativedelta(months=-NUMBER_OF_UNUSED_MONTHS_BEFORE_EXPIRY)
        expire_limit_date_str = expire_limit_date.strftime('%Y-%m-%d')
        expired_mandate_ids = self.search(cr, uid, [
            '|',
            ('last_debit_date', '=', False),
            ('last_debit_date', '<=', expire_limit_date_str),
            ('state', '=', 'valid'),
            ('signature_date', '<=', expire_limit_date_str),
            ], context=context)
        if expired_mandate_ids:
            self.write(cr, uid, expired_mandate_ids,{'state':'expired'},context=context)
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'The following SDD Mandate IDs has been set to expired: %s' % expired_mandate_ids)
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
sdd_mandate()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

